==> durt-0.1.2.tar.gz: lib/src/crypto/hd_wallet.dart <==
import 'package:durt/durt.dart';
import 'package:fast_base58/fast_base58.dart';
import 'package:hex/hex.dart';
import 'package:bip32_ed25519/api.dart';
import 'package:bip39_multi_nullsafety/bip39_multi_nullsafety.dart' as bip39;
import 'package:pinenacl/key_derivation.dart';

class HdWallet {
  final Bip32SigningKey rootSigningKey;
  Bip32KeyPair key;

  /// Root constructor taking a root signing key
  HdWallet({required this.rootSigningKey, required this.key});

  factory HdWallet.fromHexEntropy(String hexEntropy) {
    Bip32SigningKey rootSigningKeyFactory = _bip32signingKey(
      Uint8List.fromList(
        HEX.decode(hexEntropy),
      ),
    );
    return HdWallet(
        rootSigningKey: rootSigningKeyFactory,
        key: Bip32KeyPair(
            signingKey: rootSigningKeyFactory,
            verifyKey: rootSigningKeyFactory.verifyKey));
  }
  final _derivator = Bip32Ed25519KeyDerivation.instance;

  /// Build the root signing key based on given mnemonic.
  factory HdWallet.fromMnemonic(String mnemonic, {String lang = 'english'}) =>
      HdWallet.fromHexEntropy(
        bip39.mnemonicToEntropy(mnemonic.trim(), language: lang),
      );

  factory HdWallet.fromDewif(String dewif, String password) {
    String mnemonic = Dewif().mnemonicFromDewif(dewif, password);
    return HdWallet.fromMnemonic(mnemonic);
  }

  /// return the root signing key
  Bip32VerifyKey get rootVerifyKey => rootSigningKey.verifyKey;

  /// derive root signing key given a seed
  static Bip32SigningKey _bip32signingKey(Uint8List seed) {
    // final rawMaster = KeyDerivator('SHA-512/HMAC/PBKDF2')
    //   ..init(Pbkdf2Parameters(seed, 4096, cip16ExtendedSigningKeySize));
    // final Bip32SigningKey rootXsk =
    //     Bip32SigningKey.normalizeBytes(rawMaster.process(Uint8List(0)));
    final rawMaster = PBKDF2.hmac_sha512(
        Uint8List(0), seed, 4096, cip16ExtendedSigningKeySize);
    final Bip32SigningKey rootXsk = Bip32SigningKey.normalizeBytes(rawMaster);
    return rootXsk;
  }

  /// If a parent signing key is provided, a child signing key is generated. If a parent
  /// verify key is provided and the index is NOT hardened, then a child verify key is
  /// also included. If hardened and no signingKey is provied, it returns an empty pair.
  Bip32KeyPair derive(int index) {
    // computes a child extended private key from the parent extended private key.
    Bip32SigningKey? signingKey = key.signingKey != null
        ? _derivator.ckdPriv(key.signingKey!, index) as Bip32SigningKey
        : null;
    Bip32VerifyKey? verifyKey = isHardened(index)
        ? null
        : key.verifyKey != null
            ? _derivator.ckdPub(key.verifyKey!, index) as Bip32VerifyKey
            : _derivator.neuterPriv(signingKey!) as Bip32VerifyKey;
    return Bip32KeyPair(signingKey: signingKey, verifyKey: verifyKey);
  }

  /// Provide a valid signature from given derivation keypair for any message or text document.
  SignatureBase sign(String document, {required int derivation}) {
    Bip32KeyPair keypair = derive(derivation);
    return keypair.signingKey!.sign(document.codeUnits).signature;
  }

  /// Verify if the given signature is valid for the given message or text document,
  /// based on given keypair
  bool verifySign(String document, SignatureBase signature,
      {required int derivation}) {
    Bip32KeyPair keypair = derive(derivation);
    return keypair.verifyKey!.verify(
        signature: signature, message: document.codeUnits.toUint8List());
  }

  /// Return the pubkey in string format for the given derivation keypair
  String getPubkey(int derivation) {
    Bip32KeyPair subKey = derive(derivation);
    return Base58Encode(subKey.signingKey!.publicKey.rawKey.toList());
  }
}

/// Private/signing and public/varification key pair.
class Bip32KeyPair {
  final Bip32SigningKey? signingKey;
  final Bip32VerifyKey? verifyKey;
  const Bip32KeyPair({this.signingKey, this.verifyKey});
}

/// Return a 12 random worlds mnemonic based on bip39 standard.
/// You can choose a custom language like french, default is english.
String generateMnemonic({String lang = 'english'}) {
  return bip39.generateMnemonic(strength: 128, language: lang);
}

/// Hardended chain values should not have public keys.
/// They are denoted by a single quote in chain values.
const int hardenedOffset = 0x80000000;

/// Extended private key size in bytes
const cip16ExtendedSigningKeySize = 96;

/// Hardens index, meaning it won't have a public key
int harden(int index) => index | hardenedOffset;

/// Returns true if index is hardened.
bool isHardened(int index) => index & hardenedOffset != 0;

==> durt-0.1.3.tar.gz: lib/src/crypto/hd_wallet.dart <==
import 'dart:convert';

import 'package:durt/durt.dart';
import 'package:fast_base58/fast_base58.dart';
import 'package:hex/hex.dart';
import 'package:bip32_ed25519/api.dart';
import 'package:bip39_multi_nullsafety/bip39_multi_nullsafety.dart' as bip39;
import 'package:pinenacl/key_derivation.dart';

class HdWallet {
  final Bip32SigningKey rootSigningKey;
  Bip32KeyPair key;

  /// Root constructor taking a root signing key
  HdWallet({required this.rootSigningKey, required this.key});

  factory HdWallet.fromHexEntropy(String hexEntropy) {
    Bip32SigningKey rootSigningKeyFactory = _bip32signingKey(
      Uint8List.fromList(
        HEX.decode(hexEntropy),
      ),
    );
    return HdWallet(
        rootSigningKey: rootSigningKeyFactory,
        key: Bip32KeyPair(
            signingKey: rootSigningKeyFactory,
            verifyKey: rootSigningKeyFactory.verifyKey));
  }
  final _derivator = Bip32Ed25519KeyDerivation.instance;

  /// Build the root signing key based on given mnemonic.
  factory HdWallet.fromMnemonic(String mnemonic, {String lang = 'english'}) =>
      HdWallet.fromHexEntropy(
        bip39.mnemonicToEntropy(mnemonic.trim(), language: lang),
      );

  factory HdWallet.fromDewif(String dewif, String password) {
    String mnemonic = Dewif().mnemonicFromDewif(dewif, password);
    return HdWallet.fromMnemonic(mnemonic);
  }

  /// return the root signing key
  Bip32VerifyKey get rootVerifyKey => rootSigningKey.verifyKey;

  /// derive root signing key given a seed
  static Bip32SigningKey _bip32signingKey(Uint8List seed) {
    // final rawMaster = KeyDerivator('SHA-512/HMAC/PBKDF2')
    //   ..init(Pbkdf2Parameters(seed, 4096, cip16ExtendedSigningKeySize));
    // final Bip32SigningKey rootXsk =
    //     Bip32SigningKey.normalizeBytes(rawMaster.process(Uint8List(0)));
    final rawMaster = PBKDF2.hmac_sha512(
        Uint8List(0), seed, 4096, cip16ExtendedSigningKeySize);
    final Bip32SigningKey rootXsk = Bip32SigningKey.normalizeBytes(rawMaster);
    return rootXsk;
  }

  /// If a parent signing key is provided, a child signing key is generated. If a parent
  /// verify key is provided and the index is NOT hardened, then a child verify key is
  /// also included. If hardened and no signingKey is provied, it returns an empty pair.
  Bip32KeyPair derive(int index) {
    // computes a child extended private key from the parent extended private key.
    Bip32SigningKey? signingKey = key.signingKey != null
        ? _derivator.ckdPriv(key.signingKey!, index) as Bip32SigningKey
        : null;
    Bip32VerifyKey? verifyKey = isHardened(index)
        ? null
        : key.verifyKey != null
            ? _derivator.ckdPub(key.verifyKey!, index) as Bip32VerifyKey
            : _derivator.neuterPriv(signingKey!) as Bip32VerifyKey;
    return Bip32KeyPair(signingKey: signingKey, verifyKey: verifyKey);
  }

  /// Provide a valid signature from given derivation keypair for any message or text document.
  String sign(String document, {required int derivation}) {
    Bip32KeyPair keypair = derive(derivation);
    return base64Encode(
        keypair.signingKey!.sign(document.codeUnits).signature.asTypedList);
  }

  /// Verify if the given signature is valid for the given message or text document,
  /// based on given keypair
  bool verifySign(String document, String signature,
      {required int derivation}) {
    Bip32KeyPair keypair = derive(derivation);
    SignatureBase fomartedSignature = Signature(base64Decode(signature));
    return keypair.verifyKey!.verify(
        signature: fomartedSignature,
        message: document.codeUnits.toUint8List());
  }

  /// Return the pubkey in string format for the given derivation keypair
  String getPubkey(int derivation) {
    Bip32KeyPair subKey = derive(derivation);
    return Base58Encode(subKey.signingKey!.publicKey.rawKey.toList());
  }
}

/// Private/signing and public/varification key pair.
class Bip32KeyPair {
  final Bip32SigningKey? signingKey;
  final Bip32VerifyKey? verifyKey;
  const Bip32KeyPair({this.signingKey, this.verifyKey});
}

/// Return a 12 random worlds mnemonic based on bip39 standard.
/// You can choose a custom language like french, default is english.
String generateMnemonic({String lang = 'english'}) {
  return bip39.generateMnemonic(strength: 128, language: lang);
}

/// Hardended chain values should not have public keys.
/// They are denoted by a single quote in chain values.
const int hardenedOffset = 0x80000000;

/// Extended private key size in bytes
const cip16ExtendedSigningKeySize = 96;

/// Hardens index, meaning it won't have a public key
int harden(int index) => index | hardenedOffset;

/// Returns true if index is hardened.
bool isHardened(int index) => index & hardenedOffset != 0;
